Package: ADImpute
Type: Package
Title: Adaptive Dropout Imputer (ADImpute)
Version: 1.20.0
Description: Single-cell RNA sequencing (scRNA-seq) methods are typically unable
    to quantify the expression levels of all genes in a cell, creating a need
    for the computational prediction of missing values (‘dropout imputation’).
    Most existing dropout imputation methods are limited in the sense that they
    exclusively use the scRNA-seq dataset at hand and do not exploit external
    gene-gene relationship information. Here we propose two novel methods: a
    gene regulatory network-based approach using gene-gene relationships learnt
    from external data and a baseline approach corresponding to a sample-wide
    average. ADImpute can implement these novel methods and also combine them
    with existing imputation methods (currently supported: DrImpute, SAVER).
    ADImpute can learn the best performing method per gene and combine the
    results from different methods into an ensemble.
Authors@R: person("Ana Carolina", "Leote", email = "anacarolinaleote@gmail.com",
           role = c("cre","aut"), comment = c(ORCID = "0000-0003-0879-328X"))
License: GPL-3 + file LICENSE
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.0)
Imports: checkmate, BiocParallel, data.table, DrImpute, kernlab, MASS,
        Matrix, methods, rsvd, S4Vectors, SAVER, SingleCellExperiment,
        stats, SummarizedExperiment, utils
Suggests: BiocStyle, knitr, rmarkdown, testthat
biocViews: GeneExpression, Network, Preprocessing, Sequencing,
        SingleCell, Transcriptomics
RoxygenNote: 7.1.1
VignetteBuilder: knitr
BugReports: https://github.com/anacarolinaleote/ADImpute/issues
git_url: https://git.bioconductor.org/packages/ADImpute
git_branch: RELEASE_3_22
git_last_commit: 4634280
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 02:26:33 UTC; biocbuild
Author: Ana Carolina Leote [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-0879-328X>)
Maintainer: Ana Carolina Leote <anacarolinaleote@gmail.com>
